'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

class CafChunkStore {
  

  

  

  

  

  static __initStatic() {this.headerSize = 12;}

  constructor(x) {CafChunkStore.prototype.__init.call(this);
    const array = new Uint8Array(x);
    const dataView = new DataView(array.buffer);
    const type = String.fromCharCode(...array.slice(0, 4));
    const size = dataView.getBigUint64(4, false);

    this.type = type;
    this.size = size;
    this.sizeWithHeader = size + 12n;

    this.header = array.slice(0, 12);
    this.body = array.slice(12, Number(size) + 12);
  }

  get byteLength() {
    return CafChunkStore.headerSize + this.body.byteLength;
  }

  __init() {this.encode = () => {
    const h = CafChunkStore.headerSize;

    const result = new Uint8Array(this.byteLength);
    const dataView = new DataView(result.buffer);

    result.set(new TextEncoder().encode(this.type));
    dataView.setBigUint64(4, BigInt(result.byteLength - h), false);

    result.set(this.body, h);

    return result;
  };}
} CafChunkStore.__initStatic();

class CafChanChunk {
  

  

  

  

   __init() {this.type = 'chan';}

  constructor(x) {CafChanChunk.prototype.__init.call(this);CafChanChunk.prototype.__init2.call(this);
    this.channelLayoutTag = x.channelLayoutTag;
    this.channelBitmap = x.channelBitmap;
    this.numberChannelDescriptions = x.numberChannelDescriptions;
    this.channelDescriptions = x.channelDescriptions;
  }

  static __initStatic() {this.from = (data) => {
    const chunkStore = new CafChunkStore(data);

    if (chunkStore.type !== 'chan') {
      throw new TypeError(`Not a channel chunk`);
    }

    const dataView = new DataView(chunkStore.body.buffer);

    const channelLayoutTag = dataView.getUint32(0, false);
    const channelBitmap = dataView.getUint32(4, false);
    const numberChannelDescriptions = dataView.getUint32(8, false);

    const channelDescriptions = [];

    for (let i = 0; i < numberChannelDescriptions; i += 1) {
      const channelLabel = dataView.getUint32(12 + i * 12, false);
      const channelFlags = dataView.getUint32(16 + i * 12, false);

      const coordinates = [
        dataView.getFloat64(20 + i * 12, false),
        dataView.getFloat64(28 + i * 12, false),
        dataView.getFloat64(36 + i * 12, false),
      ] ;

      channelDescriptions.push({
        channelLabel,
        channelFlags,
        coordinates,
      });
    }

    return new CafChanChunk({
      channelLayoutTag,
      channelBitmap,
      numberChannelDescriptions,
      channelDescriptions,
    });
  };}

  get byteLength() {
    return (
      CafChunkStore.headerSize +
      12 +
      this.numberChannelDescriptions * (this.channelDescriptions.length * 32)
    );
  }

  __init2() {this.encode = () => {
    const h = CafChunkStore.headerSize;

    const result = new Uint8Array(this.byteLength);
    const dataView = new DataView(result.buffer);

    result.set(new TextEncoder().encode(this.type));
    dataView.setBigUint64(4, BigInt(result.length - h), false);

    dataView.setUint32(h + 0, this.channelLayoutTag, false);
    dataView.setUint32(h + 4, this.channelBitmap, false);
    dataView.setUint32(h + 8, this.numberChannelDescriptions, false);

    for (let i = 0; i < this.numberChannelDescriptions; i += 1) {
      const channelDescription = this.channelDescriptions[i];

      dataView.setUint32(
        h + 12 + i * 12,
        channelDescription.channelLabel,
        false
      );
      dataView.setUint32(
        h + 16 + i * 12,
        channelDescription.channelFlags,
        false
      );

      for (let j = 0; j < channelDescription.coordinates.length; j += 1) {
        dataView.setFloat64(
          h + 20 + i * 12 + j * 8,
          channelDescription.coordinates[j],
          false
        );
      }
    }

    return result;
  };}
} CafChanChunk.__initStatic();

class CafDataChunk {
  

  

   __init() {this.type = 'data';}

  constructor({ editCount, data }) {CafDataChunk.prototype.__init.call(this);CafDataChunk.prototype.__init2.call(this);
    this.editCount = editCount;
    this.data = data;
  }

  static __initStatic() {this.from = (data) => {
    const chunkStore = new CafChunkStore(data);

    if (chunkStore.type !== 'data') {
      throw new TypeError(`Not a data chunk`);
    }

    const dataView = new DataView(chunkStore.body.buffer);

    const editCount = dataView.getUint32(0, false);
    const body = chunkStore.body.slice(4);

    return new CafDataChunk({ editCount, data: body });
  };}

  get byteLength() {
    return CafChunkStore.headerSize + this.data.byteLength + 4;
  }

  __init2() {this.encode = () => {
    const h = CafChunkStore.headerSize;

    const result = new Uint8Array(this.byteLength);
    const dataView = new DataView(result.buffer);

    result.set(new TextEncoder().encode(this.type));
    dataView.setBigUint64(4, BigInt(result.byteLength - h), false);

    dataView.setUint32(h, this.editCount, false);
    result.set(this.data, h + 4);

    return result;
  };}
} CafDataChunk.__initStatic();

class CafDescChunk {
  

  

  

  

  

  

  

   __init() {this.type = 'desc';}

  constructor(x) {CafDescChunk.prototype.__init.call(this);CafDescChunk.prototype.__init2.call(this);
    this.sampleRate = x.sampleRate;
    this.formatID = x.formatID;
    this.formatFlags = x.formatFlags;
    this.bytesPerPacket = x.bytesPerPacket;
    this.framesPerPacket = x.framesPerPacket;
    this.channelsPerFrame = x.channelsPerFrame;
    this.bitsPerChannel = x.bitsPerChannel;
  }

  static __initStatic() {this.from = (data) => {
    const chunkStore = new CafChunkStore(data);

    if (chunkStore.type !== 'desc') {
      throw new TypeError(`Not a description chunk`);
    }

    const dataView = new DataView(chunkStore.body.buffer);

    const sampleRate = dataView.getFloat64(0, false);
    const formatID = String.fromCharCode(...chunkStore.body.subarray(8, 12));
    const formatFlags = dataView.getUint32(12, false);
    const bytesPerPacket = dataView.getUint32(16, false);
    const framesPerPacket = dataView.getUint32(20, false);
    const channelsPerFrame = dataView.getUint32(24, false);
    const bitsPerChannel = dataView.getUint32(28, false);

    return new CafDescChunk({
      sampleRate,
      formatID,
      formatFlags,
      bytesPerPacket,
      framesPerPacket,
      channelsPerFrame,
      bitsPerChannel,
    });
  };}

  // eslint-disable-next-line class-methods-use-this
  get byteLength() {
    return 44;
  }

  __init2() {this.encode = () => {
    const result = new Uint8Array(this.byteLength);
    const dataView = new DataView(result.buffer);

    result.set(new TextEncoder().encode(this.type));
    dataView.setBigUint64(4, 32n, false);

    const h = CafChunkStore.headerSize;
    dataView.setFloat64(h, this.sampleRate, false);
    result.set(new TextEncoder().encode(this.formatID), h + 8);
    dataView.setUint32(h + 12, this.formatFlags, false);
    dataView.setUint32(h + 16, this.bytesPerPacket, false);
    dataView.setUint32(h + 20, this.framesPerPacket, false);
    dataView.setUint32(h + 24, this.channelsPerFrame, false);
    dataView.setUint32(h + 28, this.bitsPerChannel, false);

    return result;
  };}
} CafDescChunk.__initStatic();

class CafHeaderChunk {
  

  

  

  constructor(x) {CafHeaderChunk.prototype.__init.call(this);
    this.fileType = x.fileType;
    this.fileVersion = x.fileVersion;
    this.fileFlags = x.fileFlags;
  }

  static __initStatic() {this.from = (x) => {
    const dataView = new DataView(x.buffer);

    return new CafHeaderChunk({
      fileType: String.fromCharCode(...x.slice(0, 4)),
      fileVersion: dataView.getUint16(4, false),
      fileFlags: dataView.getUint16(6, false),
    });
  };}

  // eslint-disable-next-line class-methods-use-this
  get byteLength() {
    return 8;
  }

  __init() {this.encode = () => {
    const result = new Uint8Array(this.byteLength);
    const dataView = new DataView(result.buffer);
    result.set(new TextEncoder().encode(this.fileType));

    dataView.setUint16(4, this.fileVersion, false);
    dataView.setUint16(6, this.fileFlags, false);

    return result;
  };}
} CafHeaderChunk.__initStatic();

class CafPaktChunk {
  

  

   __init() {this.type = 'pakt';}

  constructor({ header, body }) {CafPaktChunk.prototype.__init.call(this);CafPaktChunk.prototype.__init2.call(this);
    this.header = header;
    this.body = body;
  }

  static __initStatic() {this.packetsTable = (data) => {
    const numbers = [];
    let currentNumber = 0;

    for (let i = 0; i < data.length; i += 1) {
      const byte = data[i];
      currentNumber = (currentNumber << 7) | (byte & 0x7f);

      if ((byte & 0x80) === 0) {
        numbers.push(currentNumber);
        currentNumber = 0;
      }
    }

    return numbers;
  };}

  static __initStatic2() {this.constructPacketsTable = (numbers) => {
    const data = [];

    for (let i = 0; i < numbers.length; i += 1) {
      const encodedNumber = [];
      let number = numbers[i];

      while (number > 127) {
        const byte = number % 128;
        encodedNumber.unshift(byte);
        number = Math.floor(number / 128);
      }
      encodedNumber.unshift(number);

      for (let j = 0, l = encodedNumber.length; j < l - 1; j += 1) {
        encodedNumber[j] = encodedNumber[j] | 128;
      }

      data.push(...encodedNumber);
    }

    return new Uint8Array(data);
  };}

  static __initStatic3() {this.parsePacketTableHeader = (x) => {
    const dataView = new DataView(x.buffer);
    const numberPackets = dataView.getBigInt64(0, false);
    const numberValidFrames = dataView.getBigInt64(8, false);
    const primingFrames = dataView.getInt32(16, false);
    const remainderFrames = dataView.getInt32(20, false);

    const packetTableHeader = {
      numberPackets,
      numberValidFrames,
      primingFrames,
      remainderFrames,
    };

    return packetTableHeader;
  };}

  static __initStatic4() {this.constructParsePacketTableHeader = (
    packetTableHeader
  ) => {
    const buffer = new ArrayBuffer(24);
    const dataView = new DataView(buffer);

    dataView.setBigInt64(0, packetTableHeader.numberPackets, false);
    dataView.setBigInt64(8, packetTableHeader.numberValidFrames, false);
    dataView.setInt32(16, packetTableHeader.primingFrames, false);
    dataView.setInt32(20, packetTableHeader.remainderFrames, false);

    const reversedPacketTableHeader = new Uint8Array(buffer);

    return reversedPacketTableHeader;
  };}

  static __initStatic5() {this.from = (data) => {
    const chunkStore = new CafChunkStore(data);

    if (chunkStore.type !== 'pakt') {
      throw new TypeError(`Not a packet chunk`);
    }

    const header = chunkStore.body.slice(0, 24);
    const body = chunkStore.body.slice(24);

    return new CafPaktChunk({
      header: CafPaktChunk.parsePacketTableHeader(header),
      body: CafPaktChunk.packetsTable(body),
    });
  };}

  get byteLength() {
    const body = CafPaktChunk.constructPacketsTable(this.body);

    return CafChunkStore.headerSize + 24 + body.byteLength;
  }

  __init2() {this.encode = () => {
    const header = CafPaktChunk.constructParsePacketTableHeader(this.header);
    const body = CafPaktChunk.constructPacketsTable(this.body);

    const h = CafChunkStore.headerSize;

    const result = new Uint8Array(this.byteLength);
    const dataView = new DataView(result.buffer);

    result.set(new TextEncoder().encode(this.type));
    dataView.setBigUint64(4, BigInt(result.byteLength - h), false);

    result.set(header, h);
    result.set(body, h + header.byteLength);

    return result;
  };}
} CafPaktChunk.__initStatic(); CafPaktChunk.__initStatic2(); CafPaktChunk.__initStatic3(); CafPaktChunk.__initStatic4(); CafPaktChunk.__initStatic5();

const readCafChunkHeader = (x) => {
  const mChunkType = x.slice(0, 4);
  const mChunkSize = x.slice(4, 12);

  const header = {
    chunkType: new TextDecoder().decode(mChunkType),
    chunkSize: new DataView(mChunkSize.buffer).getBigInt64(0, false),
  };

  return header;
};

function _optionalChain$1(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }














const CAF_HEADER_CHUNK_HEADER = {
  chunkType: '#header',
  chunkSize: 8n,
};

async function* fetchCafOpusFile(
  url,
  testMode
)



 {
  
  const request = await fetch(url);
  const reader = _optionalChain$1([request, 'access', _ => _.body, 'optionalAccess', _2 => _2.getReader, 'call', _3 => _3()]);

  if (!reader) {
    throw new TypeError(`Valid reader not found`);
  }

  let done = false;
  let buffer = null;

  let headerParsed = false;

  while (!done || (buffer && buffer.length)) {
    const { value, done: d } = await reader.read();
    done = d;

    if (!buffer && value) {
      buffer = value;
    } else if (buffer && value) {
      const nextBuffer = new Uint8Array(
        buffer.length + value.length
      );
      nextBuffer.set(buffer);
      nextBuffer.set(value, buffer.length);
      buffer = nextBuffer;
    }

    if (!buffer) {
      throw new Error('Buffer not ready');
    }

    if (!headerParsed) {
      if (buffer.length < 8) {
        continue;
      }

      const headerBuffer = buffer.slice(0, 8);
      const headerChunk = CafHeaderChunk.from(headerBuffer);
      buffer = buffer.slice(8);

      headerParsed = true;

      if (testMode) {
        yield {
          header: CAF_HEADER_CHUNK_HEADER,
          chunk: headerChunk,
          raw: headerBuffer,
        } ;
      } else {
        yield headerChunk ;
      }
    }

    const header = readCafChunkHeader(buffer);
    const bodySize = Number(header.chunkSize);
    const totalSize = CafChunkStore.headerSize + bodySize;

    if (buffer.length < totalSize) {
      continue;
    }

    const raw = buffer.slice(0, totalSize);
    buffer = buffer.slice(totalSize);

    const wrapResult = (chunk) => {
      if (testMode) {
        return { header, chunk, raw } ;
      }

      return chunk ;
    };

    switch (header.chunkType) {
      case 'desc':
        yield wrapResult(CafDescChunk.from(raw));
        break;
      case 'chan':
        yield wrapResult(CafChanChunk.from(raw));
        break;
      case 'data':
        yield wrapResult(CafDataChunk.from(raw));
        break;
      case 'pakt':
        yield wrapResult(CafPaktChunk.from(raw));
        break;
      default:
        yield wrapResult(new CafChunkStore(raw));
    }
  }

  return null;
}

const oggMagicSignature = [0x4f, 0x67, 0x67, 0x53];
const opusHeadMagicSignature = [0x4f, 0x70, 0x75, 0x73, 0x48, 0x65, 0x61, 0x64];
const opusCommentMagicSignature = [
  0x4f, 0x70, 0x75, 0x73, 0x54, 0x61, 0x67, 0x73,
];

class OggFormatError extends Error {}






















class OggPage {
  __init() {this.ready = false;}

  

  

  

  

  

  

  

  

  

  

  

  

  

  

  constructor( buffer) {this.buffer = buffer;OggPage.prototype.__init.call(this);OggPage.prototype.__init2.call(this);OggPage.prototype.__init3.call(this);OggPage.prototype.__init4.call(this);
    const { dataView, pageSize, pageSegmentsCount } =
      OggPage.validatePage(buffer);

    this.dataView = dataView;
    this.structureVersion = dataView.getUint8(4);

    const headerType = dataView.getUint8(5);
    this.headerType = headerType;
    this.isFreshPacket = !(headerType & 0x1);
    this.isBos = !!((headerType & 0x2) >>> 1);
    this.isBoe = !!((headerType & 0x4) >>> 2);

    this.absoluteGranulePosition = dataView.getBigUint64(6, true);
    this.streamSerialNumber = dataView.getUint32(14, true);
    this.pageSequenceNumber = dataView.getUint32(18, true);
    this.pageChecksum = dataView.getUint32(22, true);
    this.pageSegments = pageSegmentsCount;
    this.segmentTable = buffer.slice(27, 27 + pageSegmentsCount);

    this.pageSize = pageSize;

    this.buffer = buffer.slice(0, pageSize);

    this.parsedSegmentTable = [];

    // RFC3533 Page 7
    let accumulatedSize = 0;
    for (let i = 0, l = this.segmentTable.length; i < l; i += 1) {
      const segmentSize = this.segmentTable[i];

      if (segmentSize === 255) {
        accumulatedSize += segmentSize;

        continue;
      }

      if (accumulatedSize > 0) {
        accumulatedSize += segmentSize;
        this.parsedSegmentTable.push(accumulatedSize);
        accumulatedSize = 0;

        continue;
      }

      if (segmentSize !== 0) {
        this.parsedSegmentTable.push(segmentSize);
      }
    }
  }

  __init2() {this.validatePageSize = () => {
    const headerSize = this.pageSegments + 27;
    const pageBodySize = this.segmentTable.reduce((a, b) => a + b, 0);
    const pageSize = headerSize + pageBodySize;

    if (pageSize !== this.buffer.byteLength) {
      throw new TypeError(
        `Invalid page size, expected ${pageSize}bytes but actually ${this.buffer.byteLength}bytes.`
      );
    }
  };}

  __init3() {this.getPageSegment = (i) => {
    const accumulatedPageSegmentSize = this.parsedSegmentTable
      .slice(0, i)
      .reduce((a, b) => a + b, 0);
    const segmentLength = this.parsedSegmentTable[i];

    return this.buffer.slice(
      27 + this.pageSegments + accumulatedPageSegmentSize,
      27 + this.pageSegments + accumulatedPageSegmentSize + segmentLength
    );
  };}

  __init4() {this.mapSegments = (callback) => {
    const parsedSegmentLength = this.parsedSegmentTable.length;
    const result = new Array(parsedSegmentLength);

    for (let i = 0; i < parsedSegmentLength; i += 1) {
      result[i] = callback(this.getPageSegment(i), i);
    }

    return result;
  };}

  static validatePage(buffer) {
    for (let i = 0; i < oggMagicSignature.length; i += 1) {
      if (buffer[i] !== oggMagicSignature[i]) {
        throw new OggFormatError('Invalid OGG magic signature');
      }
    }

    // OGG page header incomplete
    if (buffer.length < 27) {
      throw new OggFormatError('Incomplete buffer length');
    }
    const dataView = new DataView(buffer.buffer);

    const pageSegmentsCount = dataView.getUint8(26);
    // OGG segment table incomplete
    if (buffer.length < 27 + pageSegmentsCount) {
      throw new OggFormatError('Incomplete segment table');
    }

    let pageBodySize = 0;
    for (let i = 0; i < pageSegmentsCount; i += 1) {
      const segmentLength = dataView.getUint8(27 + i);
      pageBodySize += segmentLength;
    }

    const pageSize = 27 + pageSegmentsCount + pageBodySize;
    // OGG content not loaded completely
    if (buffer.length < pageSize) {
      throw new OggFormatError('Insufficient page size');
    }

    return { dataView, pageBodySize, pageSize, pageSegmentsCount };
  }

  getOggHeader() {
    const array = this.getPageSegment(0);
    const dataView = new DataView(array.buffer);

    for (let i = 0; i < opusHeadMagicSignature.length; i += 1) {
      if (array[i] !== opusHeadMagicSignature[i]) {
        throw new OggFormatError('Invalid magic signature');
      }
    }

    const version = dataView.getUint8(8);
    const channelCount = dataView.getUint8(9);
    const preSkip = dataView.getUint16(10, true);
    const inputSampleRate = dataView.getUint32(12, true);
    const outputGain = dataView.getUint16(16, true);
    const mappingFamily = dataView.getUint8(18);

    let channelMapping = null;

    if (array.length > 19) {
      const streamCount = dataView.getUint8(19);
      const coupledCount = dataView.getUint8(20);

      const channelMappings = [];

      for (let i = 0; i < channelCount; i += 1) {
        channelMappings.push(dataView.getUint8(21 + i));
      }

      channelMapping = {
        streamCount,
        coupledCount,
        channelMapping: channelMappings,
      };
    }

    return {
      version,
      channelCount,
      preSkip,
      inputSampleRate,
      outputGain,
      mappingFamily,
      channelMapping,
    };
  }

  getOggTags() {
    const array = this.getPageSegment(0);
    const dataView = new DataView(array.buffer);

    for (let i = 0; i < opusCommentMagicSignature.length; i += 1) {
      if (array[i] !== opusCommentMagicSignature[i]) {
        throw new Error('Invalid magic signature.');
      }
    }

    const vendorStringLength = dataView.getInt32(8, true);
    const vendorString = new TextDecoder().decode(
      array.slice(12, 12 + vendorStringLength)
    );

    const userCommentListLength = dataView.getInt32(
      12 + vendorStringLength,
      true
    );

    let commentListLengthLeft = userCommentListLength;
    const userCommentStrings = [];

    while (commentListLengthLeft > 0) {
      const offset = userCommentListLength - commentListLengthLeft;
      const userCommentStringLength = dataView.getInt32(
        12 + vendorStringLength + 4 + offset,
        true
      );
      const userCommentString = new TextDecoder().decode(
        array.slice(
          12 + vendorStringLength + 8 + offset,
          12 + vendorStringLength + 8 + offset + userCommentStringLength
        )
      );

      userCommentStrings.push(userCommentString);

      commentListLengthLeft -= userCommentStringLength + 4;
    }

    return {
      vendorString,
      userCommentString: userCommentStrings,
    };
  }

  getTimestamp(preSkip) {
    return (this.absoluteGranulePosition - BigInt(preSkip)) / BigInt(48000.0);
  }
}

exports.Mode = void 0; (function (Mode) {
  const SILK = 0; Mode[Mode["SILK"] = SILK] = "SILK";
  const Hybrid = SILK + 1; Mode[Mode["Hybrid"] = Hybrid] = "Hybrid";
  const CELT = Hybrid + 1; Mode[Mode["CELT"] = CELT] = "CELT";
})(exports.Mode || (exports.Mode = {}));

exports.BandWidth = void 0; (function (BandWidth) {
  const NB = 0; BandWidth[BandWidth["NB"] = NB] = "NB";
  const MB = NB + 1; BandWidth[BandWidth["MB"] = MB] = "MB";
  const WB = MB + 1; BandWidth[BandWidth["WB"] = WB] = "WB";
  const SWB = WB + 1; BandWidth[BandWidth["SWB"] = SWB] = "SWB";
  const FB = SWB + 1; BandWidth[BandWidth["FB"] = FB] = "FB";
})(exports.BandWidth || (exports.BandWidth = {}));

exports.FrameSize = void 0; (function (FrameSize) {
  const Ms2d5 = 2.5; FrameSize[FrameSize["Ms2d5"] = Ms2d5] = "Ms2d5";
  const Ms5 = 5; FrameSize[FrameSize["Ms5"] = Ms5] = "Ms5";
  const Ms10 = 10; FrameSize[FrameSize["Ms10"] = Ms10] = "Ms10";
  const Ms20 = 20; FrameSize[FrameSize["Ms20"] = Ms20] = "Ms20";
  const Ms40 = 40; FrameSize[FrameSize["Ms40"] = Ms40] = "Ms40";
  const Ms60 = 60; FrameSize[FrameSize["Ms60"] = Ms60] = "Ms60";
})(exports.FrameSize || (exports.FrameSize = {}));








const CONFIGS = [
  {
    index: 0,
    mode: exports.Mode.SILK,
    bandWidth: exports.BandWidth.NB,
    frameSize: exports.FrameSize.Ms10,
  },
  {
    index: 1,
    mode: exports.Mode.SILK,
    bandWidth: exports.BandWidth.NB,
    frameSize: exports.FrameSize.Ms20,
  },
  {
    index: 2,
    mode: exports.Mode.SILK,
    bandWidth: exports.BandWidth.NB,
    frameSize: exports.FrameSize.Ms40,
  },
  {
    index: 3,
    mode: exports.Mode.SILK,
    bandWidth: exports.BandWidth.NB,
    frameSize: exports.FrameSize.Ms60,
  },
  {
    index: 4,
    mode: exports.Mode.SILK,
    bandWidth: exports.BandWidth.MB,
    frameSize: exports.FrameSize.Ms10,
  },
  {
    index: 5,
    mode: exports.Mode.SILK,
    bandWidth: exports.BandWidth.MB,
    frameSize: exports.FrameSize.Ms20,
  },
  {
    index: 6,
    mode: exports.Mode.SILK,
    bandWidth: exports.BandWidth.MB,
    frameSize: exports.FrameSize.Ms40,
  },
  {
    index: 7,
    mode: exports.Mode.SILK,
    bandWidth: exports.BandWidth.MB,
    frameSize: exports.FrameSize.Ms60,
  },
  {
    index: 8,
    mode: exports.Mode.SILK,
    bandWidth: exports.BandWidth.WB,
    frameSize: exports.FrameSize.Ms10,
  },
  {
    index: 9,
    mode: exports.Mode.SILK,
    bandWidth: exports.BandWidth.WB,
    frameSize: exports.FrameSize.Ms20,
  },
  {
    index: 10,
    mode: exports.Mode.SILK,
    bandWidth: exports.BandWidth.WB,
    frameSize: exports.FrameSize.Ms40,
  },
  {
    index: 11,
    mode: exports.Mode.SILK,
    bandWidth: exports.BandWidth.WB,
    frameSize: exports.FrameSize.Ms60,
  },
  {
    index: 12,
    mode: exports.Mode.Hybrid,
    bandWidth: exports.BandWidth.SWB,
    frameSize: exports.FrameSize.Ms10,
  },
  {
    index: 13,
    mode: exports.Mode.Hybrid,
    bandWidth: exports.BandWidth.SWB,
    frameSize: exports.FrameSize.Ms20,
  },
  {
    index: 14,
    mode: exports.Mode.Hybrid,
    bandWidth: exports.BandWidth.FB,
    frameSize: exports.FrameSize.Ms10,
  },
  {
    index: 15,
    mode: exports.Mode.Hybrid,
    bandWidth: exports.BandWidth.FB,
    frameSize: exports.FrameSize.Ms20,
  },
  {
    index: 16,
    mode: exports.Mode.CELT,
    bandWidth: exports.BandWidth.NB,
    frameSize: exports.FrameSize.Ms2d5,
  },
  {
    index: 17,
    mode: exports.Mode.CELT,
    bandWidth: exports.BandWidth.NB,
    frameSize: exports.FrameSize.Ms5,
  },
  {
    index: 18,
    mode: exports.Mode.CELT,
    bandWidth: exports.BandWidth.NB,
    frameSize: exports.FrameSize.Ms10,
  },
  {
    index: 19,
    mode: exports.Mode.CELT,
    bandWidth: exports.BandWidth.NB,
    frameSize: exports.FrameSize.Ms20,
  },
  {
    index: 20,
    mode: exports.Mode.CELT,
    bandWidth: exports.BandWidth.WB,
    frameSize: exports.FrameSize.Ms2d5,
  },
  {
    index: 21,
    mode: exports.Mode.CELT,
    bandWidth: exports.BandWidth.WB,
    frameSize: exports.FrameSize.Ms5,
  },
  {
    index: 22,
    mode: exports.Mode.CELT,
    bandWidth: exports.BandWidth.WB,
    frameSize: exports.FrameSize.Ms10,
  },
  {
    index: 23,
    mode: exports.Mode.CELT,
    bandWidth: exports.BandWidth.WB,
    frameSize: exports.FrameSize.Ms20,
  },
  {
    index: 24,
    mode: exports.Mode.CELT,
    bandWidth: exports.BandWidth.SWB,
    frameSize: exports.FrameSize.Ms2d5,
  },
  {
    index: 25,
    mode: exports.Mode.CELT,
    bandWidth: exports.BandWidth.SWB,
    frameSize: exports.FrameSize.Ms5,
  },
  {
    index: 26,
    mode: exports.Mode.CELT,
    bandWidth: exports.BandWidth.SWB,
    frameSize: exports.FrameSize.Ms10,
  },
  {
    index: 27,
    mode: exports.Mode.CELT,
    bandWidth: exports.BandWidth.SWB,
    frameSize: exports.FrameSize.Ms20,
  },
  {
    index: 28,
    mode: exports.Mode.CELT,
    bandWidth: exports.BandWidth.FB,
    frameSize: exports.FrameSize.Ms2d5,
  },
  {
    index: 29,
    mode: exports.Mode.CELT,
    bandWidth: exports.BandWidth.FB,
    frameSize: exports.FrameSize.Ms5,
  },
  {
    index: 30,
    mode: exports.Mode.CELT,
    bandWidth: exports.BandWidth.FB,
    frameSize: exports.FrameSize.Ms10,
  },
  {
    index: 31,
    mode: exports.Mode.CELT,
    bandWidth: exports.BandWidth.FB,
    frameSize: exports.FrameSize.Ms20,
  },
];

CONFIGS.map((x) => Object.freeze(x));

exports.Channel = void 0; (function (Channel) {
  const Mono = 0; Channel[Channel["Mono"] = Mono] = "Mono";
  const Stereo = 1; Channel[Channel["Stereo"] = Stereo] = "Stereo";
})(exports.Channel || (exports.Channel = {}));

class OpusParseError extends Error {}

const parseDoubleByteFrameLength = (byte1, byte2) => {
  return byte2 * 4 + byte1;
};

const parseFrameLengthCoding = (dataView, index) => {
  if (dataView.byteLength <= index + 1) {
    throw new OpusParseError(
      `Unable to parse the frame length, insufficient data length (expected at least ${index} bytes, got ${dataView.byteLength}), check: https://datatracker.ietf.org/doc/html/rfc6716#section-3.2.1`
    );
  }

  const result = parseDoubleByteFrameLength(
    dataView.getUint8(index),
    dataView.getUint8(index + 1)
  );

  return result;
};










const parseOpusPacket = (x) => {
  const tocByte = x[0];
  const configLabel = (tocByte & 0b11111000) >>> 3;

  const dataView = new DataView(x.buffer);

  const config = CONFIGS[configLabel];

  const channel = ((tocByte & 0b00000100) >>> 2) ;

  const framesFlag = tocByte & 0b00000011;

  let frames = 0;
  let hasPadding = false;
  let paddingLength = 0;
  let frameLengthByteLength = 0;
  const frameByteLengths = [];

  if (framesFlag === 0) {
    // Implements https://datatracker.ietf.org/doc/html/rfc6716#section-3.2.2
    frames = 1;
    frameByteLengths.push(x.byteLength - 1);
  } else if (framesFlag === 1) {
    // Implements https://datatracker.ietf.org/doc/html/rfc6716#section-3.2.3
    frames = 2;

    if ((x.byteLength - 1) % 2 === 1) {
      throw new OpusParseError(
        `Code 1 packets MUST have an odd total length, but got a ${x.byteLength}bytes packet, check: https://datatracker.ietf.org/doc/html/rfc6716#ref-R3`
      );
    }

    frameByteLengths.push((x.byteLength - 1) / 2);
  } else if (framesFlag === 2) {
    frames = 2;

    if (x.byteLength === 1) {
      throw new OpusParseError(
        `A 1-byte code 2 packet is invalid, see: https://datatracker.ietf.org/doc/html/rfc6716#section-3.2.4`
      );
    }

    frameLengthByteLength = dataView.getUint8(1) > 251 ? 2 : 1;

    if (x.byteLength === 2 && frameLengthByteLength === 2) {
      throw new OpusParseError(
        `A 2-byte code 2 packet whose second byte is in the range 252...255 is invalid, see: https://datatracker.ietf.org/doc/html/rfc6716#section-3.2.4`
      );
    }
    if (frameLengthByteLength === 2) {
      // Implements https://datatracker.ietf.org/doc/html/rfc6716#section-3.2.4
      const firstFrameByteLength = parseFrameLengthCoding(dataView, 1);
      frameByteLengths.push(
        firstFrameByteLength,
        x.byteLength - 3 - firstFrameByteLength
      );
    } else if (frameLengthByteLength === 1) {
      const firstFrameByteLength = dataView.getUint8(1);
      frameByteLengths.push(
        firstFrameByteLength,
        x.byteLength - 2 - firstFrameByteLength
      );
    } else {
      throw new OpusParseError(`Unexpected frame length byte length.`);
    }
  } else if (framesFlag === 3) {
    if (x.byteLength < 2) {
      throw new OpusParseError(
        `Code 3 packets MUST have at least 2 bytes, check: https://datatracker.ietf.org/doc/html/rfc6716#ref-R6`
      );
    }
    const frameCountByte = x[1];
    frames = frameCountByte & 0b00111111;
    hasPadding = !!((frameCountByte & 0b01000000) >>> 6);
    const isCBR = !!((frameCountByte & 0b10000000) >>> 7);

    if (frames > 48) {
      throw new OpusParseError(
        `The maximum frame count for any code 3 is 48, but got ${frameCountByte}, check: https://datatracker.ietf.org/doc/html/rfc6716#ref-R5`
      );
    }

    if (frames === 0) {
      throw new OpusParseError(
        `Code 3 packets contain at least one frame, check: https://datatracker.ietf.org/doc/html/rfc6716#ref-R5`
      );
    }

    if (hasPadding) {
      paddingLength = dataView.getUint8(2);

      if (paddingLength === 255) {
        paddingLength = 254 + dataView.getUint8(3);
      }
    }

    if (isCBR) {
      // p is the number of header bytes used to indicate the padding size plus the number of padding bytes themselves.
      const p = (paddingLength > 254 ? 2 : 1) + paddingLength;
      // In the CBR case, let R=N-2-P be the number of bytes remaining in the packet after subtracting the (optional) padding.
      const r = x.byteLength - 2 - p;
      const frameSize = r / frames;

      if (frameSize !== Math.round(frameSize)) {
        throw new OpusParseError(
          `The value R MUST be a non-negative integer multiple of M (R=${r}, M=${frames}), check: https://datatracker.ietf.org/doc/html/rfc6716#ref-R6`
        );
      }

      for (let i = 0; i < frames; i += 1) {
        frameByteLengths.push(frameSize);
      }
    } else {
      for (let i = 0; i < frames; i += 1) {
        // Implemented https://datatracker.ietf.org/doc/html/rfc6716#section-3.2.1
        const beginIndex = 2 + frameLengthByteLength;

        if (x.byteLength <= beginIndex) {
          throw new OpusParseError(
            `Failed to parse frame byte length for frame #${i}, insufficient data length (expected at least ${beginIndex} bytes, got ${dataView.byteLength}), see: https://datatracker.ietf.org/doc/html/rfc6716#section-3.2.1`
          );
        }

        let frameByteLength = 0;
        const thisFrameLengthByteLength =
          dataView.getUint8(beginIndex) > 251 ? 1 : 2;

        if (thisFrameLengthByteLength === 2) {
          frameByteLength = parseFrameLengthCoding(dataView, beginIndex);
        } else if (thisFrameLengthByteLength === 1) {
          frameByteLength = dataView.getUint8(beginIndex);
        } else {
          throw new OpusParseError(`Unexpected frame length byte length.`);
        }

        frameByteLengths.push(frameByteLength);
        frameLengthByteLength += thisFrameLengthByteLength;
      }
    }
  } else {
    throw new OpusParseError(`Invalid frames flag.`);
  }

  return {
    config,
    channel,
    framesFlag,
    paddingLength,
    frameLengthByteLength,
    frameByteLengths,
  };
};

function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }















async function* fetchOggOpusFile(url) {
  const request = await fetch(url);
  const reader = _optionalChain([request, 'access', _ => _.body, 'optionalAccess', _2 => _2.getReader, 'call', _3 => _3()]);

  if (!reader) {
    throw new TypeError(`Valid reader not found`);
  }

  let done = false;
  let buffer = null;

  while (!done || (buffer && buffer.length)) {
    const { value, done: d } = await reader.read();
    done = d;

    if (!buffer && value) {
      buffer = value;
    } else if (buffer && value) {
      const nextBuffer = new Uint8Array(
        buffer.length + value.length
      );
      nextBuffer.set(buffer);
      nextBuffer.set(value, buffer.length);
      buffer = nextBuffer;
    }

    if (!buffer) {
      throw new Error('Buffer not ready');
    }

    const page = new OggPage(buffer);

    try {
      page.validatePageSize();
    } catch (e) {
      if (e instanceof OpusParseError) {
        continue;
      }

      throw e;
    }

    if (page.pageSequenceNumber === 0) {
      const result = {
        type: 'header',
        page,
        data: page.getOggHeader(),
      };

      yield result;
    }

    if (page.pageSequenceNumber === 1) {
      const result = {
        type: 'tags',
        page,
        data: page.getOggTags(),
      };

      yield result;
    }

    if (page.pageSequenceNumber > 1) {
      const packets = page.mapSegments(parseOpusPacket);

      const result = {
        type: 'packets',
        page,
        data: packets,
      };

      yield result;
    }

    buffer = buffer.slice(page.pageSize);
  }

  return null;
}

const OGG_CHANNEL_COUNT_TO_CAF_CHANNEL_LAYOUT_TAG = [
  (100 << 16) | 1, // kCAFChannelLayoutTag_Mono
  (101 << 16) | 2, // kCAFChannelLayoutTag_Stereo
  (113 << 16) | 3, // kCAFChannelLayoutTag_MPEG_3_0_A (This is actually incorrect, OGG defines it as L C R, but CAF do not have such a tags)
  (108 << 16) | 4, // kCAFChannelLayoutTag_Quadraphonic
  (119 << 16) | 5, // kCAFChannelLayoutTag_MPEG_5_0_C
];






async function* oggOpusToCaf(
  x


,
  testMode
)



 {
  

  const wrapResult = (c) => {
    return testMode ? ({ chunk: c, raw: c.encode() } ) : (c );
  };

  yield wrapResult(
    new CafHeaderChunk({
      fileType: 'caff',
      fileVersion: 1,
      fileFlags: 0,
    })
  );

  let done = false;
  let value



 = null;

  const packetsParseResults = [];
  let sampleRate = -1;

  let header = null;

  while (!done) {
    const { done: d, value: v } = await x.next();

    done = !!d;
    value = v;

    if (!value) continue;

    if (value.type === 'header') {
      header = value;
      continue;
    }

    if (value.type === 'packets') {
      packetsParseResults.push(value);
    }
  }

  if (header && packetsParseResults.length) {
    const firstPacket = packetsParseResults[0];

    sampleRate = header.data.inputSampleRate;

    yield wrapResult(
      new CafDescChunk({
        sampleRate: header.data.inputSampleRate,
        formatID: 'opus',
        formatFlags: 0,
        bytesPerPacket: 0,
        framesPerPacket:
          (firstPacket.data[0].config.frameSize * header.data.inputSampleRate) /
          1000,
        channelsPerFrame: header.data.channelCount,
        bitsPerChannel: 0,
      })
    );

    yield wrapResult(
      new CafChanChunk({
        channelLayoutTag:
          OGG_CHANNEL_COUNT_TO_CAF_CHANNEL_LAYOUT_TAG[
            header.data.channelCount - 1
          ],
        channelBitmap: 0,
        numberChannelDescriptions: 0,
        channelDescriptions: [],
      })
    );
  }

  const totalDataSize = packetsParseResults
    .flatMap((y) => y.page.parsedSegmentTable)
    .reduce((a, b) => a + b, 0);

  const dataChunk = new CafDataChunk({
    editCount: 0,
    data: new Uint8Array(totalDataSize),
  });

  let offset = 0;
  for (let i = 0, l = packetsParseResults.length; i < l; i += 1) {
    const packetsParseResult = packetsParseResults[i];
    const { page } = packetsParseResult;
    for (let j = 0, ll = page.parsedSegmentTable.length; j < ll; j += 1) {
      const chunkData = page.getPageSegment(j);
      dataChunk.data.set(chunkData, offset);
      offset += chunkData.length;
    }
  }

  yield wrapResult(dataChunk);

  const numberPackets = packetsParseResults
    .map((y) => y.page.parsedSegmentTable.length)
    .reduce((a, b) => a + b, 0);

  if (sampleRate === -1) {
    throw new Error(
      'Sample rate not found in header, unable to calculate numberValidFrames'
    );
  }

  const numberValidFrames = packetsParseResults
    .flatMap((y) => y.data.map((z) => (z.config.frameSize * sampleRate) / 1000))
    .reduce((a, b) => a + b, 0);

  const body = packetsParseResults.flatMap((y) =>
    y.page.parsedSegmentTable.flatMap((packetSize) => {
      return [packetSize];
    })
  );

  return wrapResult(
    new CafPaktChunk({
      header: {
        numberPackets: BigInt(numberPackets),
        numberValidFrames: BigInt(numberValidFrames),
        primingFrames: 0,
        remainderFrames: 0,
      },

      body,
    })
  );
}

exports.CafChanChunk = CafChanChunk;
exports.CafChunkStore = CafChunkStore;
exports.CafDataChunk = CafDataChunk;
exports.CafDescChunk = CafDescChunk;
exports.CafHeaderChunk = CafHeaderChunk;
exports.CafPaktChunk = CafPaktChunk;
exports.OggFormatError = OggFormatError;
exports.OggPage = OggPage;
exports.OpusParseError = OpusParseError;
exports.fetchCafOpusFile = fetchCafOpusFile;
exports.fetchOggOpusFile = fetchOggOpusFile;
exports.oggOpusToCaf = oggOpusToCaf;
exports.parseOpusPacket = parseOpusPacket;
exports.readCafChunkHeader = readCafChunkHeader;
//# sourceMappingURL=main.js.map
